import java.util.*;


abstract class Func<U, V> {
	
	public abstract V apply(U param);
	
	public <T> Func<T, V> compose(
			final Func<T, U> other) {
		return new Func<T, V>() {

			@Override
			public V apply(T param) {
				return Func.this.apply(
						other.apply(param));
			}
		};
	}
}

public class Compose {
	
	public static void main(String[] args) {
		Func<List<Integer>, Integer> head =
				new Func<List<Integer>, Integer>() {

					@Override
					public Integer apply(List<Integer> param) {
						return param.get(0);
					}
			
		};
		
		Func<List<Integer>, List<Integer>> tail =
				new Func<List<Integer>, List<Integer>>() {

					@Override
					public List<Integer> apply(List<Integer> param) {
						List<Integer> copy =
								new ArrayList<Integer>(param);
						copy.remove(0);
						return copy;
					}
	
		};
		
		List<Integer> l = Arrays.asList(1, 2, 3);
		
		System.out.println(head.apply(l)); // head(l);
		System.out.println(tail.apply(l)); // tail(l);
		
		Func<List<Integer>, Integer> headTail = head.compose(tail);
		System.out.println(headTail.apply(l)); // (head . tail)(l);
	}
}