; ----- Bool -----

(define T-0 (λ (x y) x))
(define F-0 (λ (x y) y))

(define not-0
  (λ (x)
    (x F-0 T-0)))

(display "not-0\n")
(not-0 T-0)
(not-0 F-0)

(define-macro and-0 ; evaluare normală
  (λ (x y)
    `((,x (λ () ,y) (λ () F-0)))))

(display "and-0\n")
(and-0 T-0 T-0)
(and-0 F-0 T-0)
(and-0 T-0 F-0)
(and-0 F-0 F-0)

(define-macro or-0 ; evaluare normală
  (λ (x y)
    `((,x (λ () T-0) (λ () ,y)))))

(display "or-0\n")
(or-0 T-0 T-0)
(or-0 F-0 T-0)
(or-0 T-0 F-0)
(or-0 F-0 F-0)

(define-macro if-0 ; evaluare normală
  (λ (c t e)
    `((,c (λ () ,t) (λ () ,e)))))

(display "if-0\n")
(if-0 T-0 not-0 F-0)
(if-0 F-0 not-0 F-0)


; ----- Pair -----

(define pair-0
  (λ (x y)
    (λ (s)
      (s x y))))

(define p1 (pair-0 T-0 F-0))

(define fst-0
  (λ (p)
    (p T-0)))

(display "fst-0\n")
(fst-0 p1)

(define snd-0
  (λ (p)
    (p F-0)))

(display "snd-0\n")
(snd-0 p1)


; ----- List -----

(define null-0 (λ (x) T-0))
(define cons-0 pair-0)

(define car-0 fst-0)
(define cdr-0 snd-0)
(define null?-0 (λ (L) (L (λ (x y) F-0))))

(define L1 (cons-0 T-0 (cons-0 F-0 null-0)))

(define list-0->std
  (λ (L)
    (if-0 (null?-0 L) '()
          (cons (car-0 L) (list-0->std (cdr-0 L))))))

(display "list-0->std\n")
(list-0->std L1)

(define std->list-0
  (λ (L)
    (if (null? L) null-0
          (cons-0 (car L) (std->list-0 (cdr L))))))

(define L2 (std->list-0 (list T-0 not-0 fst-0)))

(display "list-0->std o std->list-0\n")
(list-0->std L2)


; ----- Natural -----

(define zero-0 null-0)
(define succ-0 (λ (n) (cons-0 'whatever n)))
(define pred-0 cdr-0)
(define zero?-0 null?-0)
;(define add-0 append-0)

(define nat-0->std
  (λ (n)
    (if-0 (zero?-0 n) 0
        (+ 1 (nat-0->std (pred-0 n))))))

(define std->nat-0
  (λ (n)
    (if (zero? n) zero-0
        (succ-0 (std->nat-0 (- n 1))))))

(define n1 (std->nat-0 5))

(display "nat-0->std o std->nat-0\n")
(nat-0->std n1)


; ----- Recursivitate -----

(define Fix
  (λ (f)
    ((λ (x) (f (x x))) (λ (x) (f (x x))))))

(define Length-0 ; funcţionala care induce length-0
  (λ (f)
    (λ (L)
      (if-0 (null?-0 L) zero-0
            (succ-0 (f (cdr-0 L)))))))

;(define length-0 (Fix Length-0))
; (Fix Length-0) -> (Length (Fix Length-0)) -> (Length (Length (Fix Length-0))) -> ...
; Crapă datorită evaluării aplicative.

; Pentru funcţii unare
(define c1
  (λ (f)
    ((λ (g) (λ (x) ((f (g g)) x)))
     (λ (g) (λ (x) ((f (g g)) x))))))

(define length-0 (c1 Length-0))
; (Fix Length-0) -> λx.((Length (Fix Length-0)) x)
; Expresie echivalentă comportamental cu cea de mai sus, dar în FNF, deci evaluarea, fie ea aplicativă, se opreşte.

(display "length-0\n")
(nat-0->std (length-0 L2))

; Pentru funcţii binare
(define c2
  (λ (f)
    ((λ (g) (λ (x y) ((f (g g)) x y)))
     (λ (g) (λ (x y) ((f (g g)) x y))))))

(define Append-0 ; funcţionala care induce append-0
  (λ (f)
    (λ (L1 L2)
      (if-0 (null?-0 L1) L2
            (cons-0 (car-0 L1) (f (cdr-0 L1) L2))))))

(define append-0 (c2 Append-0))

(display "list-0->std o append-0\n")
(list-0->std (append-0 L2 L2))

; Variantă ce foloseşte c1, pentru funcţii unare, deoarece al doilea parametru rămâne nemodificat.
(define append-0-c1
  (λ (L1 L2)
    ((c1
      (λ (f)
        (λ (L)
          (if-0 (null?-0 L) L2
                (cons-0 (car-0 L) (f (cdr-0 L)))))))
     L1)))

(display "list-0->std o append-0-c1\n")
(list-0->std (append-0-c1 L2 L2))
