(include "prod-abstractions.rkt")

(define-macro stream-cons
  (lambda (head tail)
    `(cons ,head (pack ,tail))))

(define stream-car car)

(define stream-cdr
  (lambda (s)
    (unpack (cdr s))))

(define stream-null '())

(define stream-null? null?)

(define stream-take
  (lambda (n s)
    (cond [(zero? n) '()]
          [(stream-null? s) '()]
          [else (cons (stream-car s)
                      (stream-take
                       (- n 1)
                       (stream-cdr s)))])))

(define stream-drop
  (lambda (n s)
    (cond [(zero? n) s]
          [(stream-null? s) s]
          [else (stream-drop (- n 1)
                             (stream-cdr s))])))

(define stream-map
  (lambda (f s)
    (if (stream-null? s) s
        (stream-cons
         (f (stream-car s))
         (stream-map f (stream-cdr s))))))

(define stream-filter
  (lambda (f? s)
    (cond [(stream-null? s) s]
          [(f? (stream-car s))
           (stream-cons
            (stream-car s) 
            (stream-filter f? (stream-cdr s)))]
          [else (stream-filter
                 f?
                 (stream-cdr s))])))

(define stream-zip-with
  (lambda (f s1 s2)
    (if (stream-null? s1) s2
        (stream-cons
         (f (stream-car s1) (stream-car s2)) 
         (stream-zip-with
          f
          (stream-cdr s1)
          (stream-cdr s2))))))


(define stream-append
  (lambda (s1 s2)
    (if (stream-null? s1) s2
        (stream-cons (stream-car s1)
                     (stream-append
                      (stream-cdr s1) 
                      s2)))))

(define list->stream
  (lambda (L)
    (if (null? L) stream-null
        (stream-cons (car L)
                     (list->stream (cdr L))))))